/********************************** (C) COPYRIGHT *******************************
* File Name          : Main.c
* Author             : kinsley@WCH
* Version            : V1.0
* Date               : 2021/05/27
* Description 		 : SPI0 DMA շ
*******************************************************************************/

#include "CH57x_common.h"

#include "stdbool.h"
#include "stdint.h"




/*CS    PA12
  SCK0  PA13
  MOSI  PA14
  MISO  PA15
*/
void DebugInit(void)		
{
    GPIOA_SetBits(GPIO_Pin_9);
    GPIOA_ModeCfg(GPIO_Pin_8, GPIO_ModeIN_PU);
    GPIOA_ModeCfg(GPIO_Pin_9, GPIO_ModeOut_PP_5mA);
    UART1_DefInit();
}






typedef enum {
    SPI_MASTER_EVENT_DONE, ///< Transfer done.
} spi_master_evt_t;

typedef struct {
    unsigned char   * p_rx_buffer; ///< Pointer to RX buffer.
    unsigned short  rx_length;     ///< RX buffer length.
} spi_xfer_desc_t;

typedef struct {
    spi_master_evt_t  type;      ///< Event type.
    spi_xfer_desc_t xfer_desc; ///< Transfer details.
} spi_evt_t;



typedef void (* spi_master_evt_handler_t)(spi_evt_t const * p_event);

static spi_master_evt_handler_t m_spi_handles = NULL;

void spi_master_init(spi_master_evt_handler_t  handler) {
    SPI0_MasterDefInit();
    SPI0_CLKCfg(4);
    m_spi_handles = handler;
    NVIC_EnableIRQ( SPI0_IRQn );
}

void spi_master_send_data(uint8_t *data,uint16_t length) {
    R32_PA_DIR |= GPIO_Pin_14; //MOSIл
    R8_SPI0_CTRL_MOD &= ~RB_SPI_FIFO_DIR; //out  
    R16_SPI0_TOTAL_CNT = length;    
                                      
    R16_SPI0_DMA_END = (UINT32)(data + length);
    R16_SPI0_DMA_BEG = (UINT32)data;     
    
    R8_SPI0_INT_FLAG = RB_SPI_IF_CNT_END|RB_SPI_IF_DMA_END;

    R8_SPI0_CTRL_CFG |= RB_SPI_DMA_ENABLE;

    R8_SPI0_INTER_EN |= RB_SPI_IE_CNT_END;
}






void spi_master_recieve_data(uint8_t *data,uint16_t length){
    R32_PA_DIR &= ~GPIO_Pin_14; //MOSIлΪ,issue here
    R8_SPI0_CTRL_MOD |= RB_SPI_FIFO_DIR; //in 
    R16_SPI0_TOTAL_CNT = length;                                         
    R16_SPI0_DMA_END = (UINT32)(data + length);
    R16_SPI0_DMA_BEG = (UINT32)data;     
    R8_SPI0_INT_FLAG = RB_SPI_IF_CNT_END|RB_SPI_IF_DMA_END;
    R8_SPI0_CTRL_CFG |= RB_SPI_DMA_ENABLE;
    R8_SPI0_INTER_EN |= RB_SPI_IE_CNT_END;
}



void spi_master_send_data_blocking(uint8_t *data,uint16_t length) {
    R32_PA_DIR |= GPIO_Pin_14; //MOSIл
    R8_SPI0_CTRL_MOD &= ~RB_SPI_FIFO_DIR; //out  
    R16_SPI0_TOTAL_CNT = length;    
                                      
    R16_SPI0_DMA_END = (UINT32)(data + length);
    R16_SPI0_DMA_BEG = (UINT32)data;     
    
    R8_SPI0_INT_FLAG = RB_SPI_IF_CNT_END|RB_SPI_IF_DMA_END;

    R8_SPI0_CTRL_CFG |= RB_SPI_DMA_ENABLE;
    
    while(!(R8_SPI0_INT_FLAG & RB_SPI_IF_CNT_END));
    R8_SPI0_CTRL_CFG &= ~RB_SPI_DMA_ENABLE;
    //R8_SPI0_INTER_EN |= RB_SPI_IE_CNT_END;
}

void spi_master_recieve_data_blocking(uint8_t *data,uint16_t length){
    R32_PA_DIR &= ~GPIO_Pin_14; //MOSIлΪ,issue here
    R8_SPI0_CTRL_MOD |= RB_SPI_FIFO_DIR; //in 
    R16_SPI0_TOTAL_CNT = length;                                         
    R16_SPI0_DMA_END = (UINT32)(data + length);
    R16_SPI0_DMA_BEG = (UINT32)data;     
    R8_SPI0_INT_FLAG = RB_SPI_IF_CNT_END|RB_SPI_IF_DMA_END;
    R8_SPI0_CTRL_CFG |= RB_SPI_DMA_ENABLE;
    while(!(R8_SPI0_INT_FLAG & RB_SPI_IF_CNT_END));
    R8_SPI0_CTRL_CFG &= ~RB_SPI_DMA_ENABLE;
}

void SPI0_IRQHandler(void){
    R8_SPI0_INTER_EN = 0;
    R8_SPI0_CTRL_CFG &= ~RB_SPI_DMA_ENABLE;
    spi_evt_t evt;
    evt.type = SPI_MASTER_EVENT_DONE;
    m_spi_handles(&evt);

}


static bool spi_is_going = false;

void spi_dma_evt_handler(spi_evt_t const * p_event){
    switch(p_event->type){
        case SPI_MASTER_EVENT_DONE:
            spi_is_going = false;
            //PRINT("spi dma done \r\n");
            break;
        default:
            break;
    }
}


__align(4) uint8_t test_data[12]={1,2,3,4,5,6,7,8,9,0,1,2};
__align(4) uint8_t test_data2[12]={1,2,3,4,5,6,7,8,9,0,1,2};

int main()
{
    UINT8 i;
    
/* ôڵ */   
    DebugInit();
    PRINT( "Start @ChipID=%02X\n", R8_CHIP_ID );
    
/* ģʽ */ 
    PRINT( "1.spi0 mul master mode send data ...\n");   
    DelayMs( 100 );
    
    SetSysClock(CLK_SOURCE_HSE_32MHz);
        
    HClk32M_Select(Clk32M_HSE);
    
    //init spi0 gpio
    GPIOA_SetBits( GPIO_Pin_12 );
    GPIOA_ModeCfg(GPIO_Pin_1|GPIO_Pin_12|GPIO_Pin_13|GPIO_Pin_14, GPIO_ModeOut_PP_5mA);
    
    //init spi0
    spi_master_init(spi_dma_evt_handler);
    //custom cs for spi0
    GPIOA_SetBits(GPIO_Pin_1);
    
    while(1){
        PRINT("tx\r\n");
        
        
        spi_is_going = true;
        //custom cs for spi0
        GPIOA_ResetBits(GPIO_Pin_1);
        spi_master_send_data(test_data,12);
        //wating flag
        while(true == spi_is_going);
        //custom cs for spi0
        GPIOA_SetBits(GPIO_Pin_1);
        
        

        spi_is_going = true;
        GPIOA_ResetBits(GPIO_Pin_1);
        spi_master_recieve_data(test_data2,12);
        //wating flag
        while(true == spi_is_going);
        //custom cs for spi0
        GPIOA_SetBits(GPIO_Pin_1);
        
        
        
        for(uint8_t i =0;i<12;i++){
            PRINT("%02x ",test_data2[i]);    
        }
        PRINT("\r\n");
        
        
        GPIOA_ResetBits(GPIO_Pin_1);
        spi_master_send_data_blocking(test_data,12);
        //custom cs for spi0
        GPIOA_SetBits(GPIO_Pin_1);
        
        GPIOA_ResetBits(GPIO_Pin_1);
        spi_master_recieve_data_blocking(test_data2,12);
        //custom cs for spi0
        GPIOA_SetBits(GPIO_Pin_1);
        
        
        
        for(uint8_t i =0;i<12;i++){
            PRINT("%02x ",test_data2[i]);    
        }
        PRINT("\r\n");
        mDelaymS(100);
    }   
     
}






